'use strict';

var core = require('echarts-comp/core');
var React = require('react');

const EChartsComp = React.forwardRef(function ECharts(props, ref) {
    const { domRef, echarts } = useECharts(props);
    React.useImperativeHandle(ref, () => echarts, [echarts]);
    return React.createElement("div", { style: props.style, ref: domRef });
});
/** When the theme or initOption changes (shallow comparison), reinitialize Echarts */
function useECharts(config = {}) {
    const { fixedSize, loading, option } = config;
    const domRef = React.useRef(null);
    const chartRef = React.useRef();
    const realTheme = useCompareValue(config.theme);
    const realInitOption = useCompareValue(config.init);
    React.useLayoutEffect(refreshOption, [option]);
    React.useLayoutEffect(refreshLoading, [loading]);
    React.useLayoutEffect(() => {
        if (!domRef.current)
            return console.error("Unable to get DOM instance");
        chartRef.current?.dispose();
        chartRef.current = core.init(domRef.current, realTheme, realInitOption);
        refreshOption();
        refreshLoading();
        return () => chartRef.current?.dispose();
    }, [realTheme, realInitOption]);
    const onWindowResize = React.useCallback((ev) => chartRef.current?.resize(), []);
    //resize
    React.useLayoutEffect(() => {
        if (fixedSize)
            window.removeEventListener("resize", onWindowResize);
        else
            window.addEventListener("resize", onWindowResize);
        return () => window.removeEventListener("resize", onWindowResize);
    }, [fixedSize]);
    function refreshOption() {
        if (!chartRef.current)
            return;
        chartRef.current.setOption(option ?? {}, true, false);
    }
    function refreshLoading() {
        const chart = chartRef.current;
        if (!chart)
            return;
        if (loading)
            chart.showLoading();
        else
            chart.hideLoading();
    }
    return { domRef, echarts: chartRef.current };
}
function useCompareValue(value) {
    const before = React.useRef(value);
    return React.useMemo(() => {
        if (!objectIsEqual(before.current, value))
            before.current = value;
        return before.current;
    }, [value]);
}
function objectIsEqual(obj1, obj2) {
    if ((obj1 === null && obj2 === null) || typeof obj1 !== "object" || typeof obj2 !== "object")
        return obj1 === obj2;
    const obj1Map = new Map(Object.entries(obj1));
    const obj2List = Object.entries(obj2);
    if (obj1Map.size !== obj2List.length)
        return false;
    for (let i = 0; i < obj2List.length; i++) {
        const [k2, v2] = obj2List[i];
        if (obj1Map.has(k2)) {
            if (v2 !== obj1Map.get(k2))
                return false;
            obj1Map.delete(k2);
        }
        else
            return false;
    }
    return true;
}
const ECharts = React.memo(EChartsComp);

exports.ECharts = ECharts;
exports.useECharts = useECharts;
